clear all; clc;%close all;

Ts=0.1; % timestep for integration
sim_time=100; %total simulation time

%% Model Parameters
A=0.1; %area of Level tank
Kp=0.002; %

%% Kalman Filter Parameters
Q=[0.01 0;0 0.0001];
R=0.000001;
P_initial=eye(2);

%% Initial condition
h(1)=0.5;
F_out(1)=0.004; 
X_init=h;
u=2*ones(1,sim_time/Ts);
SP=0.5*ones(1,sim_time/Ts);
d=0.004*ones(1,sim_time/Ts);

%% Cases
% Controller parameters
 Kc=10;
 Ti=10;



%% a. Calculation of Kalman Gain
    
cho=menu('Choose a case','Step in OutFlow','Step in Set-point');
    if cho==1
    t = Ts:Ts:sim_time;  
    step = t>=50;
    S_S=input('Step size for the change in out flow (-0.004 to 0.004) ');
    d=d+step*S_S;
end
   
if cho==2
    S_S=input('Step size for the change in setpoint (-0.5 to 0.5) ');
    t = Ts:Ts:sim_time;  
    step = t>=50;
    SP=SP+step*S_S;
end

% Defining the system
A_cont=[0 -1/A; 0 0];
B_cont=[Kp/A;0];
C_cont=[1 0];
D_cont=[0];
sys_cont=ss(A_cont,B_cont,C_cont,D_cont);
sys_dis = c2d(sys_cont,Ts);
A_dis=sys_dis.a;
B_dis=sys_dis.b;
C_dis=sys_dis.c;
y=model_est(Ts,sim_time, Kp, A, Kc, Ti, SP, u, d, X_init, A_dis, B_dis,C_dis, Q, R, P_initial);
K=y(3:4,end)

%% Plot the results
subplot(3,1,1)
plot(y(1,:),'r');
hold on
plot(SP,'--');
hold on
plot(y(5,:),'-b');
xlabel('Time (s)');
ylabel('Level (m)');
legend('Estimated Level','Set Point','Level');
axis([0 sim_time/Ts 0 1]);

subplot(3,1,2)
plot(d);
hold on
plot(y(2,:),'-r');
xlabel('Time (s)');
ylabel('Flow (m3/s)');
legend('F_o_u_t','F_o_u_t_e_s_t');
axis([0 sim_time/Ts 0 0.01])

subplot(3,1,3)
plot(y(6,:));
xlabel('Time (s)');
ylabel('Control Signal');

axis([0 sim_time/Ts 0 6])

function y=model_est(Ts,sim_time, Kp, A, Kc, Ti, SP, u, d, X_init, A_dis, B_dis,C_dis, Q, R, P_initial)

n_time=sim_time/Ts;
X(1)=X_init;
Xest(:,1)=[X_init;d(1)];
h_meas(1)=X(1);

int=0;
P=P_initial;
 
for i=2:n_time
    
    % Process Update------------------------
    X(i)=(Kp/A*u(i-1)-1/A*d(i-1))*Ts+X(i-1);
    h(i)=X(i);
    %---------------------------------------
  
    % Measurement Noise---------------------
    noise=0.003;
    h_meas(i)=h(i)+noise*rand;
    %---------------------------------------
    
    % Controller Update---------------------
    error=SP(i-1)-h_meas(i-1);
    int=int+error*Ts;
    u(i)=Kc*error+Kc/Ti*int;
    %---------------------------------------
  
    
    % Kalman Filter-----------------------------------------------------
    % predict
    Xest_ap=A_dis*Xest(:,i-1)+B_dis*u(i);
    P=A_dis*P*A_dis'+Q;
    
    % update
    y=h_meas(i)-C_dis*Xest_ap;
    S=R+C_dis*P*C_dis';
    K(:,i)=P*C_dis'*inv(S);
    Xest(:,i)=Xest_ap+K(:,i)*y;
    P=(eye(2)-K(:,i)*C_dis)*P*(eye(2)-K(:,i)*C_dis)'+K(:,i)*R*K(:,i)';
    %-------------------------------------------------------------------
    
    lvl=Xest(1,:);
    Fout=Xest(2,:);
    
end
y=[lvl;Fout;K;h;u];
end


